# Copyright (c) 2020 - 2020 TomTom N.V.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import sys
from typing import Any, List, Mapping

import hopic.build

log = logging.getLogger(__name__)


def _kebabify(name: str):
    return name.replace('_', '-')


def _kwarg_to_arg(name, value):
    if value is True:
        return [f"--{_kebabify(name)}"]
    elif value is not False and value is not None:
        return [f"--{_kebabify(name)}", str(value)]
    else:
        return []


def _kwargs_to_args(**kwargs: Mapping[str, Any]):
    for key, val in kwargs.items():
        yield from _kwarg_to_arg(key, val)


def _commisery_command(*ranges: List[str], **kwargs):
    return (sys.executable, '-m', 'commisery.checking', *_kwargs_to_args(**kwargs), '--', *ranges)


def commisery(volume_vars: Mapping[str, str], *, require_ticket: bool = False):
    hopic_git_info = hopic.build.HopicGitInfo.from_repo(volume_vars['WORKSPACE'])

    if hopic_git_info.target_commit and hopic_git_info.autosquashed_commits:
        check_command = {
            'image': None,
            'description': "Checking all commits provided in the Pull Request",
            'sh': _commisery_command(
                f'{hopic_git_info.target_commit}..{hopic_git_info.autosquashed_commits[0]}',
                ticket=require_ticket,
            ),
        }
    else:
        check_command = {
            'image': None,
            'foreach': 'AUTOSQUASHED_COMMIT',
            'sh': (sys.executable, '-m', 'commisery.checking', '${AUTOSQUASHED_COMMIT}'),
        }

    return [
        check_command,
        {
            'description': "Checking merge commit. The subject and content of which may originate from your Pull Request's title and description",
            'sh': _commisery_command('HEAD', ticket=False),
        }
    ]



# Copyright (c) 2020 - 2020 TomTom N.V. (https://tomtom.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import sys
import types

import git
import pytest
from click.testing import CliRunner

try:
    # Python >= 3.8
    from importlib import metadata
except ImportError:
    import importlib_metadata as metadata


commisery_range_cli = [ep for ep in metadata.entry_points()['console_scripts'] if ep.name == 'commisery-verify-msg'][0].load()


@pytest.fixture()
def commisery_cli():
    def _cli(commit_messages, rev_range=None, with_ticket=False, with_tags=(), verbose=True):
        runner = CliRunner(mix_stderr=False)
        with runner.isolated_filesystem():
            with git.Repo.init() as repo:
                if isinstance(commit_messages, types.GeneratorType):
                    commit_messages = list(commit_messages)
                for msg in commit_messages:
                    repo.index.commit(msg, author=git.Actor('Bob', 'bob@tester.org>'))

                if rev_range is None:
                    # We'll check all messages if revision range is not provided
                    rev_range = ('HEAD',)

                assert isinstance(rev_range, collections.abc.Sequence)

                args = ['-v', 'DEBUG'] if verbose else []
                if with_ticket:
                    args.extend(('-j',))
                if with_tags:
                    args.extend(('-t', ','.join(with_tags)))
                args.extend(rev_range)

                result = runner.invoke(commisery_range_cli, args=args, color=True)

                if result.stdout_bytes:
                    print(result.stdout, end='')
                if result.stderr_bytes:
                    print(result.stderr, end='', file=sys.stderr)

                if result.exception is not None and not isinstance(result.exception, SystemExit):
                    raise result.exception

                return result

    return _cli



# Copyright (c) 2018 - 2019 TomTom N.V. (https://tomtom.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import namedtuple
import re
import typing


_Footer = namedtuple('Footer', ('token', 'value'))


class CommitMessage(object):
    line_separator = '\n'
    paragraph_separator = '\n\n'
    autosquash_re = re.compile(r'^(?:(?:fixup|squash)!\s+)+')
    merge_re = re.compile(r'^Merge.*?:[ \t]*')

    # Variation of conventional commits footer that more closely matches 'git trailers'.
    # In particular it doesn't permit 'BREAKING CHANGE' (with a space, instead of '-') as a footer's token.
    footer_re = re.compile(r'''
    # 8.  One or more footers MAY be provided one blank line after the body. ...
    \n

    # 8.  ... Each footer MUST consist of a word token, ...
    (?P<token>

    # 9.  A footer's token MUST use `-` in place of whitespace characters, e.g. `Acked-by` (this helps differentiate
    #     the footer section from a multi-paragraph body). ...
        \w+(?:-\w+)*
    )

    # 8.  ..., followed by either a `: ` or ` #` separator, followed by a string value (this is inspired by the git
    #     trailer convention).
    (?::[ ]|[ ](?=[#]))
    ''', re.VERBOSE)

    def __init__(self, message, hexsha=None):
        if isinstance(message, str):
            self.message = _strip_message(message)
        else:
            self.message = _strip_message(message.message)
            try:
                if hexsha is None:
                    hexsha = message.hexsha
            except AttributeError:
                pass
        if hexsha is not None:
            self.hexsha = hexsha

        # Discover starts of lines
        self._line_index = [m.end() for m in re.finditer(self.line_separator, self.message)]
        self._line_index.insert(0, 0)
        if len(self._line_index) < 2 or self._line_index[-1] < len(self.message):
            self._line_index.append(len(self.message) + 1)

        autosquash = self.autosquash_re.match(self.message)
        self._autosquash_end = autosquash.end() if autosquash is not None else 0

        merge = self.merge_re.match(self.message[self._autosquash_end:])
        self._subject_start = self._autosquash_end + (merge.end() if merge is not None else 0)

        # Discover starts of paragraphs
        self._paragraph_index = [m.end() for m in re.finditer(self.paragraph_separator, self.message)]
        if not self.message[self._line_index[1] - len(self.line_separator):].startswith(self.paragraph_separator):
            self._paragraph_index.insert(0, self._line_index[1])
        self._paragraph_index.append(len(self.message) + len(self.paragraph_separator))

        # Strip last line terminator from the last paragraph.
        if self.message and self.message[-1] == self.line_separator:
            self._paragraph_index[-1] -= 1

        self._footer_index = [(m.group('token'), m.start(), m.end()) for m in self.footer_re.finditer(self.message)]

    @property
    def lines(self):
        return _IndexedList(self.message, self._line_index, self.line_separator)

    @property
    def full_subject(self):
        return self.message[:self._line_index[1] - 1]

    @property
    def subject_start(self):
        return self._subject_start

    @property
    def subject(self):
        return self.full_subject[self.subject_start:]

    @property
    def autosquash_end(self):
        return self._autosquash_end

    def needs_autosquash(self):
        return self.autosquash_end > 0

    @property
    def autosquashed_subject(self):
        return self.full_subject[self.autosquash_end:]

    @property
    def body(self):
        return self.message[self._paragraph_index[0]:]

    @property
    def paragraphs(self):
        return _IndexedList(self.message, self._paragraph_index, self.paragraph_separator)

    def paragraph_line(self, idx):
        if idx < 0:
            idx += len(self._paragraph_index) - 1
        idx = self._paragraph_index[idx]
        return self.message[:idx].count(self.line_separator)

    @property
    def footers(self):
        return _FooterList(self.message, self._footer_index)

    def has_breaking_change(self):
        return None

    def has_new_feature(self):
        return None

    def has_fix(self):
        return None

    def __repr__(self):
        try:
            return f"{self.__class__.__name__}({self.message!r}, {self.hexsha!r})"
        except AttributeError:
            return f"{self.__class__.__name__}({self.message!r})"


class ConventionalCommit(CommitMessage):
    strict_subject_re = re.compile(r'''
    ^
    # 1. Commits MUST be prefixed with a type, which consists of a noun, feat, fix, etc., ...
    (?P<type_tag>\w+)

    # 4. A scope MAY be provided after a type. A scope MUST consist of a noun describing a section of the codebase
    #    surrounded by parenthesis, e.g., `fix(parser):`
    (?: \( (?P<scope> [^()]* ) \) )?

    # 1. Commits MUST be prefixed with a type, ..., followed by ..., OPTIONAL `!`, ...
    (?P<breaking>\s*!(?:\s+(?=:))?)?

    # 1. Commits MUST be prefixed with a type, ..., and REQUIRED terminal colon and space.
    (?P<separator>:?[ ]?)

    # 5. A description MUST immediately follow the colon and space after the type/scope prefix. The description is a
    #    short description of the code changes, e.g., fix: array parsing issue when multiple spaces were contained in
    #    string.
    (?P<description>.*)
    $
    ''', re.VERBOSE)

    footer_re = re.compile(r'''
    # 8.  One or more footers MAY be provided one blank line after the body. ...
    \n

    # 8.  ... Each footer MUST consist of a word token, ...
    (?P<token>

    # 9.  A footer's token MUST use `-` in place of whitespace characters, e.g. `Acked-by` (this helps differentiate
    #     the footer section from a multi-paragraph body). ...
        \w+(?:-\w+)*

    # 9.  ... An exception is made for `BREAKING CHANGE`, which MAY
    #     also be used as a token.
    | BREAKING[ ]CHANGE
    )

    # 8.  ..., followed by either a `: ` or ` #` separator, followed by a string value (this is inspired by the git
    #     trailer convention).
    (?::[ ]|[ ](?=[#]))
    ''', re.VERBOSE)

    def __init__(self, message, hexsha=None):
        super().__init__(message, hexsha=hexsha)
        m = self.strict_subject_re.match(self.subject)
        if not m:
            raise RuntimeError(f"commit message's subject ({self.subject!r}) not formatted according to Conventional Commits ({self.strict_subject_re.pattern})")
        self.type_tag     = m.group('type_tag')
        self.scope        = m.group('scope')
        self._is_breaking = m.group('breaking')
        self.description  = m.group('description')

        if re.search(r'[A-Z]', self.type_tag):
            raise RuntimeError("commit message's type tag ({self.type_tag!r}) contains upper case letters but isn't allowed to".format(self=self))

        if self.scope is not None and not re.match(r'^\S+(?:.*\S+)?$', self.scope):
            raise RuntimeError("commit message's scope ({self.scope!r}) is empty or contains excess whitespace but shouldn't".format(self=self))

        if self._is_breaking is not None and self._is_breaking != '!':
            raise RuntimeError("breaking change indicator in commit message's subject should be exactly '!' (have: {self._is_breaking!r})".format(self=self))

        if m.group('separator') != ': ':
            raise RuntimeError("commit message's subject lacks a ': ' separator after the type tag (subject={self.subject!r})".format(self=self))

        if not self.description:
            raise RuntimeError("commit message's subject lacks a description after the type tag (subject={self.subject!r})".format(self=self))

        # 10. A footer's value MAY contain spaces and newlines, and parsing MUST terminate when the next valid footer
        #     token/separator pair is observed.
        self._footer_index = [(m.group('token'), m.start(), m.end()) for m in self.footer_re.finditer(self.message)]

    @property
    def footers(self):
        return _ConventionalFooterList(self.message, self._footer_index)

    def has_breaking_change(self):
        if self._is_breaking:
            return True

        for token, value in self.footers:
            if token == 'BREAKING CHANGE':
                return True

        return False

    def has_new_feature(self):
        return self.type_tag.lower() == 'feat'

    def has_fix(self):
        return self.type_tag.lower() == 'fix'


def parse_commit_message(message, policy=None, strict=False):
    if policy == 'conventional-commits':
        try:
            return ConventionalCommit(message)
        except RuntimeError:
            if strict:
                raise
    return CommitMessage(message)


class _IndexedList(object):
    def __init__(self, message, index, separator):
        self._message = message
        self._index = index
        self.separator = separator

    def __len__(self):
        return len(self._index) - 1

    def __getitem__(self, idx):
        if idx < 0:
            idx += len(self)
        return self._message[self._index[idx] : self._index[idx+1] - len(self.separator)]


class _FooterList(object):
    def __init__(self, message, index):
        self._message = message
        self._index = index

    def __len__(self):
        return len(self._index)

    def __getitem__(self, idx):
        if isinstance(idx, str):
            matches = [footer.value for footer in self if footer.token.casefold() == idx.casefold()]
            if not matches:
                raise KeyError(f"{idx} not found in footer list")
            return matches

        if idx < 0:
            idx += len(self)
        token, token_start, content_start = self._index[idx]
        content_end = self._index[idx+1][1] if idx + 1 < len(self) else len(self._message)
        while content_end > content_start and self._message[content_end-1] == '\n':
            content_end -= 1

        # 16. `BREAKING-CHANGE` MUST be synonymous with `BREAKING CHANGE`, when used as a token in a footer.
        if token == 'BREAKING-CHANGE':
            token = 'BREAKING CHANGE'

        return _Footer(token=token, value=self._message[content_start:content_end])

    def get(self, key : str, default=()) -> typing.Sequence:
        if not isinstance(key, str):
            raise TypeError(f"Only 'str' keys are supported, '{type(key).__name__}' passed instead")
        try:
            return self[key]
        except KeyError:
            return default


class _ConventionalFooterList(_FooterList):
    def __getitem__(self, idx):
        # 16. `BREAKING-CHANGE` MUST be synonymous with `BREAKING CHANGE`, when used as a token in a footer.
        if isinstance(idx, str):
            if idx.casefold() == 'BREAKING-CHANGE'.casefold():
                idx = 'BREAKING CHANGE'
            return super().__getitem__(idx)
        else:
            footer = super().__getitem__(idx)
            if footer.token == 'BREAKING-CHANGE':
                return _Footer('BREAKING CHANGE', footer.value)
            else:
                return footer


def _strip_message(message):
    cut_line = message.find('# ------------------------ >8 ------------------------\n')
    if cut_line >= 0 and (cut_line == 0 or message[cut_line - 1] == '\n'):
        message = message[:cut_line]

    # Strip comments
    message = re.sub(r'^#[^\n]*\n?', '', message, flags=re.MULTILINE)
    # Strip trailing whitespace from lines
    message = re.sub(r'[ \t]+$', '', message, flags=re.MULTILINE)
    # Merge consecutive empty lines into a single empty line
    message = re.sub(r'(?<=\n\n)\n+', '', message)
    # Remove empty lines from the beginning and end
    while message[:1] == '\n':
        message = message[1:]
    while message[-2:] == '\n\n':
        message = message[:-1]

    return message



# Copyright (c) 2020 - 2020 TomTom N.V. (https://tomtom.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .. import checking
import pytest
import re
import sys


def strip_terminal_ctrl_codes(msg):
    return re.sub(r'\x1B\[(?:\d+(?:;\d+)*)?m', '', msg)


def get_verification_failures(msg, nr=None):
    err = '\n'.join(checking.check_commit_message('test-commit', msg))
    sys.stderr.write(err)

    diagnostics = []
    line_re = re.compile(r'test-commit:\d+:\d+: (?:error|note): ')
    for line in strip_terminal_ctrl_codes(err).splitlines(keepends=True):
        if line_re.match(line):
            diagnostics.append(line)
        else:
            if not diagnostics:
                print((line, line_re), file=sys.stderr)
            diagnostics[-1] += line

    return diagnostics


def test_initial_capital():
    diagnostic, = get_verification_failures('fix: Change the thingy')
    assert re.search(r'\btitle case\b.*\bdescription\b', diagnostic)


@pytest.mark.parametrize('msg', (
    'fix: change as per review comments',
    'fix: process review comment',
    'fix: use different DLT contexts in Positioning Service - process review comment',
    'fix: introduce ServiceSimulator tool: resolve review findings',
    'fix: use systemd to start the navigationwindowmanager (review comments)',
    'fix: resolve review comments',
    'fix: implement DLT logging in Positioning Service (review comments)',
    'fix: changes according review',
    'fix: processing minor code review comments',
    'fix: implemented review comments',
    'fix: processed all review comments',
    'fix: process review comments (rename version file and remove trailing spaces)',
    'fix: process review comment',
    'fix: gNSS Simulator. Fixes for review comments by Jitendra',
    'fix: code review comments resolved',
    'fix: processed minor review comment',
    'fix: processed review comments for blabla',
    'fix: apply review comments',
    'fix: addressd review comments',
    'fixup! fix: ATF_TestCasesNeedFix_RoadShields test cases - incorporated review comments',
    'fix: edit online with Bitbucket',
    'fix: apply reviewer\'s comments',
))
def test_reject_referring_to_review_comments(msg):
    error, note = get_verification_failures(msg)
    assert re.search(r'\bcontext\b.*\binstead\b.*\brefer.*\breview.*\b(?:comment|message)', error)
    assert '--fixup' in note


@pytest.mark.parametrize('msg', (
    'fix: force XML encoding to UTF-8',
    'fix: check computed SHA-256 hash against expected value',
    'feat: display colors using VT-100/VT-220 control codes',
    'feat: implement AES-128 CTR mode en/decryption',
    'fix: ensure generated version numbers adhere to PEP-440',
))
def test_accept_common_abbreviations_as_not_Jira(msg):
    assert not get_verification_failures(msg)


def test_accept_capital_TomTom():
    assert not get_verification_failures('feat: TomTom home screen redecorated')


def test_type_tag_typo():
    diagnostic, = get_verification_failures('impovemtn: verify the thingamajig')
    assert diagnostic.splitlines()[-1] == 'improvement'


def test_scope_whitespace():
    error, = get_verification_failures('improvement( verifier ): verify the thingamajig')
    assert re.search(r'\bwhite.*space\b.*\bscope\b', error)
    assert not get_verification_failures('improvement(thingamajig verifier): verify the thingamajig')


def test_empty_scope():
    error, = get_verification_failures('improvement(): verify the thingamajig')
    assert re.search(r'\bscope\b.*\bempty\b', error)


def test_missing_separator():
    error, = get_verification_failures('improvement verify the thingamajig')
    assert re.search(r'\bseparator\b.*\btype\b.*\btag\b', error)


def test_description_whitespace():
    error, = get_verification_failures('improvement: verify the   thingamajig')
    assert re.search(r'\bwhite.*space\b.*\bdescription\b', error)


def test_jira_ticket():
    error, = get_verification_failures('improvement: verify the thingamajig introduced with PIPE-423')
    assert re.search(r'\bcontains?\b.*\bjira tickets?\b', error, re.IGNORECASE)


def test_punctuation():
    error, = get_verification_failures('improvement: verify the thingamajig.')
    assert 'punctuation' in error


def test_body_unseparated_from_subject():
    error, = get_verification_failures('''\
improvement: verify the thingamajig
Hmpf
Bla bla...
''')
    assert re.search(r'\bsubject\b.*\bbody\b.*\bseparat.*\bempty line\b', error)


def test_non_imperative():
    error, note = get_verification_failures('fix: added missing files')
    assert re.search(r'\bdescription\b.*\b(?:blacklisted|imperative)\b', error)
    assert 'imperative' in note


def test_whitespace_breaking():
    error, = get_verification_failures('refactor(mooh) !: revamp API to be more awesome')
    assert re.search(r'\bbreaking change indicator\b.*\b(?:whitespace|should\b.*\bexactly)\b', error)


def test_review_comment_spread_out_in_body():
    error, note = get_verification_failures('''feat: something awesome

Perform
some review
rework.

Implements #PIPE-123''')

    assert re.search(r'\bcontext\b.*\binstead\b.*\brefer.*\breview.*\b(?:comment|message)', error)
    assert '--fixup' in note



# Copyrighy (c) 2020 - 2020 TomTom N.V. (https://tomtom.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from textwrap import dedent

import pytest

from commisery.checking import DEFAULT_ACCEPTED_TAGS

log = logging.getLogger(__name__)


def test_cli_range_of_correct_long_commits(commisery_cli):
    long_commit_messages = tuple(dedent(f'''chore: write a long message for this test

        This should be a proper commit body. Nice and long, too.
        Other test cases use more terse commit messages, this one tests a long one.
        It also references UTF-8 for no apparent reason, it doesn't comply with PEP-440,
        is represented by a SHA-1 commit hash and was written on a VT-52 terminal..

        Okay, maybe not that last one, but the rest is mostly accurate.

        Implements: TICKET-{x}
        ''') for x in range(0, 10))

    result = commisery_cli(long_commit_messages)
    assert result.exit_code == 0

    result = commisery_cli(long_commit_messages, with_tags=('foo', 'bar'))
    assert result.exit_code == 1

    result = commisery_cli(long_commit_messages, with_tags=('chore',), with_ticket=True)
    assert result.exit_code == 0


def test_cli_no_conventional_commit(commisery_cli):
    result = commisery_cli(('not a conventional commit message',))
    assert result.exit_code == 1

    result = commisery_cli(('fix the issue',))
    assert result.exit_code == 1


def test_cli_error_msg_for_no_ticket(commisery_cli, caplog):
    result = commisery_cli(commit_messages=('chore: commit without ticket',),
                           rev_range=('HEAD', 'HEAD'),
                           with_ticket=True, verbose=False)
    assert r'No ticket is referenced within the provided range of commits' in caplog.text
    assert result.exit_code == 1


@pytest.mark.parametrize('rev_range, expected_exit_code', (
    (('HEAD',   '^HEAD~1'),      0),
    (('HEAD~1', '^HEAD~3'),      1),
    (('HEAD~3', '^HEAD~4'),      0),
    (('HEAD~4', '^HEAD~5'),      1),
    (('HEAD~6', 'HEAD~6'),       0),
))
def test_cli_rev_ranges(commisery_cli, rev_range, expected_exit_code):
    commits = ('fix: this is a syntactically correct commit message',
               'incorrect commit: number four',
               'incorrect commit: number three',
               'fix: this is a syntactically correct commit message',
               'incorrect commit: number two',
               'incorrect commit: number one',
               'fix: this is a syntactically correct commit message',
               )

    result = commisery_cli(commit_messages=commits, rev_range=rev_range)
    assert result.exit_code == expected_exit_code


@pytest.mark.parametrize('commits, with_tags, expected_exit_code', (
        (('chore: correct commit', 'fix: correct commit'),   (),                   0),
        (('mytag: correct commit', 'ci: correct commit'),    ('mytag', 'docs'),    1),
        (('mytag: correct commit', 'ci: correct commit'),    ('mytag', 'ci'),      0),
        (('mytag: correct commit', 'mytag: another one'),    ('mytag',),           0),
        (('fix: this is a fix', 'fox: this is a fox'),       ('fox',),             0),  # 'fix' and 'feat' are
        (('feat: this is a feat', 'foot: this is a foot'),   ('foot',),            0),  # always allowed
        (('fix: yep ok', 'notmytag: incorrect tag here',),   ('mytag',),           1),
        (('chore: correct commit', 'fix: correct commit'),   ('myspecialtag',),    1),
        (('chore: correct commit', 'fix: correct commit'),   ('chore', 'fix'),     0),
))
def test_cli_with_accepted_tags(commisery_cli, commits, with_tags, expected_exit_code):
    result = commisery_cli(commit_messages=commits, rev_range=('HEAD', 'HEAD'), with_tags=with_tags)
    assert result.exit_code == expected_exit_code


@pytest.mark.parametrize('rev_range', (('HEAD',), ('HEAD', 'HEAD')))
@pytest.mark.parametrize('commit', (tag for tag in DEFAULT_ACCEPTED_TAGS))
def test_cli_default_tags(commisery_cli, rev_range, commit):
    result = commisery_cli(commit_messages=(f'{commit}: correct commit',), rev_range=rev_range)
    assert result.exit_code == 0


def test_single_commit_wrong_tag(commisery_cli):
    result = commisery_cli(commit_messages=(f'wibble: wrong tag',), rev_range=('HEAD',))
    assert result.exit_code == 1


@pytest.mark.parametrize('rev_range', (('HEAD',), ('HEAD', 'HEAD')))
@pytest.mark.parametrize('commits, expected_exit_code', (
        (('fix: this one has a TICKET-1 in the title so it will fail anyway',),      1),
        (('ci: correct parent commit', 'ci: correct commit\n\nAddresses: TICK-1'),   0),
        (('ci: this one does not reference a ticket',),                              1),
        (('ci: this one references UTF-8 in the title, but no ticket',),             1),
        (('ci: commit without ticket\n\nAddresses: UTF-8\nAddresses: SHA-256',),     1),
        (('ci: referencing two tickets\n\nAddresses: TIC-1\nImplements: KET-2',),    0),
))
def test_cli_ticket_present(commisery_cli, rev_range, commits, expected_exit_code):
    result = commisery_cli(commit_messages=commits, rev_range=rev_range, with_ticket=True)
    assert result.exit_code == expected_exit_code



#!/usr/bin/env python3

# Copyright (C) 2020-2020, TomTom (http://tomtom.com).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click
import tempfile

from github import Github
from . import checking


def check_message(message: str) -> bool:
    with tempfile.NamedTemporaryFile() as tmp:
        tmp.write(message.encode('UTF-8'))
        tmp.flush()

        return checking.check_commit(tmp.name) == 0


@click.command()
@click.option('-t', '--token',
              required=True, help='GitHub Token')
@click.option('-r', '--repository',
              required=True,  help='GitHub repository')
@click.option('-p', '--pull-request-id',
              required=True, help='Pull Request identifier')
def main(token: str, repository: str, pull_request_id: int) -> int:
    errors = 0

    repo = Github(token).get_repo(repository)
    pr = repo.get_pull(int(pull_request_id))

    if not check_message(pr.title):
        errors += 1

    commits = pr.get_commits()

    for commit_info in commits:
        if not check_message(commit_info.commit.message):
            errors += 1

    exit(1 if errors else 0)


if __name__ == '__main__':
    main()
